package com.nisum.productconsumer.service;

import com.nisum.productconsumer.model.Product;
import com.nisum.productconsumer.repository.ProductRepository;
import org.reactivestreams.Publisher;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.List;

@Service
public class ProductService {

    @Autowired
    private ProductRepository productRepository;

    private static final Logger logger= LoggerFactory.getLogger(ProductService.class);

    @KafkaListener(topics = "product-topic", groupId = "product-group1")
    public void productConsume(List<Product> products) {

        Flux<Product> productFlux = Flux.fromIterable(products);

        productFlux
                .doOnNext(product -> logger.info("Consumed Product: {}", product))
                .flatMap(product -> saveProduct(product))  // Save to database
                .subscribe(
                        product -> logger.info("Saved Product: {}", product),
                        error -> logger.error("Error saving product: {}", error.getMessage())
                );
    }

    private Mono<Product> saveProduct(Product product) {
        return productRepository.save(product)
                .doOnSuccess(savedProduct -> logger.info("Successfully saved product: {}", savedProduct))
                .doOnError(error -> logger.error("Failed to save product: {}", error.getMessage()));
    }
}
